<?php
/*
  Payment module for National Australia Bank's NAB Transact Payment Gateway (Hosted Payment Page)
  
  Copyright (c) 2008 I2 Technology Solutions
  www.i2ts.com.au
  development@i2ts.com.au
  
  Released under the GNU General Public License
    
  Version 1.4 
*/
  //Define internal constants
  //These constants are defined here (instead of /includes/languages/english/modules/payment/) 
  //The lanuage file will not be available for install() function. Hence the 
  //constants can not be used in install() 
  define('MODULE_PAYMENT_NAB_TRANSACT_CONST_YES', 'Yes');
  define('MODULE_PAYMENT_NAB_TRANSACT_CONST_NO', 'No'); 
  define('MODULE_PAYMENT_NAB_TRANSACT_CONST_TEST', 'Test');
  define('MODULE_PAYMENT_NAB_TRANSACT_CONST_LIVE', 'Live'); 
    
  define('MODULE_PAYMENT_NAB_TRANSACT_HPP_CONST_LIVE_URL', 'https://transact.nab.com.au/live/hpp/payment');
  define('MODULE_PAYMENT_NAB_TRANSACT_HPP_CONST_TEST_URL', 'https://transact.nab.com.au/test/hpp/payment');

  define('MODULE_PAYMENT_NAB_TRANSACT_QS_PARAM_NAME_BANK_REF', 'bank_reference');
  define('MODULE_PAYMENT_NAB_TRANSACT_QS_PARAM_NAME_CARD_TYPE', 'card_type');
  define('MODULE_PAYMENT_NAB_TRANSACT_QS_PARAM_NAME_AMNT_PAID', 'payment_amount');
  define('MODULE_PAYMENT_NAB_TRANSACT_QS_PARAM_NAME_REF_NUM', 'payment_number');
  define('MODULE_PAYMENT_NAB_TRANSACT_QS_PARAM_NAME_CUST_IP', 'remote_ip');
  
  define('MODULE_PAYMENT_NAB_TRANSACT_CLIENT_ID_LENGTH', 7);
  
  define('MODULE_PAYMENT_NAB_TRANSACT_ERR_UNAUTH_CHECKOUT_CALL', '01');
  define('MODULE_PAYMENT_NAB_TRANSACT_ERR_INVALID_MERCH_ID', '02');
  
  class nab_transact_hpp {
    var $code, $title, $public_title, $description, $enabled;
    //var $payment_class->email_footer //Interesting reference to var found in checkout_process.php (sending email) Testmore
    
    // constructor
    function nab_transact_hpp() {
      global $order, $messageStack;

      $this->code = 'nab_transact_hpp';
      $this->enabled = (MODULE_PAYMENT_NAB_TRANSACT_HPP_ENABLED == MODULE_PAYMENT_NAB_TRANSACT_CONST_YES);
      $this->form_action_url = ($this->in_test_mode() ? MODULE_PAYMENT_NAB_TRANSACT_HPP_CONST_TEST_URL : MODULE_PAYMENT_NAB_TRANSACT_HPP_CONST_LIVE_URL);
      $this->sort_order = MODULE_PAYMENT_NAB_TRANSACT_HPP_SORT_ORDER;
      $this->title = MODULE_PAYMENT_NAB_TRANSACT_HPP_TEXT_ADMIN_TITLE;
      //Add warning/error msg in Admin
      //IS_ADMIN_FLAG is defined in language file
      //this flag is used in language file as as well. As the language file is loaded before the module
      //this flag is defined in language filr rather than here
      if (IS_ADMIN_FLAG === true) {
        if ($this->enabled) {
          if ($errors = $this->config_errors()) {      
            $this->title .= ' <span class="errorText">'.MODULE_PAYMENT_NAB_TRANSACT_HPP_ERROR_CONFIG_SHORT.'</span>';
            $this->description .= '<span class="errorText">'.MODULE_PAYMENT_NAB_TRANSACT_HPP_ERROR_CONFIG.'<br/><br/>';
            foreach($errors as $err_code)
            {
              $error_array = $this->get_error($err_code, 'short');
              $this->description .= '* '.$error_array['error'].'<br/>';      
            }
            $this->description .= '</span><br/>'.tep_draw_separator();
          } elseif ($warnings = $this->warnings()) {
            $this->title .= ' <span class="messageStackWarning">'.MODULE_PAYMENT_NAB_TRANSACT_HPP_WARN_SHORT.'</span>';
            $this->description .= '<span class="messageStackWarning">'.MODULE_PAYMENT_NAB_TRANSACT_HPP_WARN.'<br/><br/>';
            foreach($warnings as $warning) {
              $this->description .= '* '.$warning.'<br/>';
            }
            $this->description .= '</span><br/>'.tep_draw_separator();
          }
        }
      }
      //Add the warning to the cart
      if ($warnings = $this->warnings()) {
        foreach($warnings as $warning) {
          $messageStack->add('header', $warning, 'warning');
        }
      }
      $this->public_title = MODULE_PAYMENT_NAB_TRANSACT_HPP_TEXT_PUBLIC_TITLE;
      $this->description .= MODULE_PAYMENT_NAB_TRANSACT_HPP_TEXT_DESCRIPTION;
      if ($this->enabled) {
        $this->description .= MODULE_PAYMENT_NAB_TRANSACT_HPP_TEXT_TESTING_INFO;
      } else {
        $this->description .= MODULE_PAYMENT_NAB_TRANSACT_HPP_TEXT_REQUIREMENTS;
      }
      
      if (is_object($order)) $this->update_status();
    }

    //class methods
    
    //Update the status of the module
    //This function is used to decide if this payment method will be avialable or not
    //depending on various criteria such as customer's country, shipping method, etc.. 
    //This function is called by checkout confirmation process
    function update_status() {
      //@todo:implement zone and shipping based control
      //Shipping based control is requierd for example don't provide credit card payments for store pick-up
      //the merchant can select to which zones and shipping methods they want to make this payment module available
      //possible way of building this:
      //display pair of shipping and zone lists in the Admin 
      //One include list, another exclude list
      //merchant can select multiple zones and shipping methods this payment method should be available from the include list
      //merchant can select multiple zones and shipping methods this payment method shouldn't be available from the exclude list
      //providing a list rather than just a drop-down box will allow the merchants to select multiple items rather than just one  
      
      global $order;

      if ( ($this->enabled == true) && ((int)MODULE_PAYMENT_NAB_TRANSACT_HPP_ZONE > 0) ) {
        $check_flag = false;
        $check_query = tep_db_query("select zone_id from " . TABLE_ZONES_TO_GEO_ZONES . " where geo_zone_id = '" . MODULE_PAYMENT_NAB_TRANSACT_HPP_ZONE . "' and zone_country_id = '" . $order->billing['country']['id'] . "' order by zone_id");
        while ($check = tep_db_fetch_array($check_query)) {
          if ($check['zone_id'] < 1) {
            $check_flag = true;
            break;
          } elseif ($check['zone_id'] == $order->billing['zone_id']) {
            $check_flag = true;
            break;
          }
        }

        if ($check_flag == false) {
          $this->enabled = false;
        }
      }
    }
    
    //Javascript required to validate the payment form returned by selection()
    //For HPP this will return false as there is no form to validate (card details are entered in HPP)
    //Called by the shopping cart while listing the available payment options
    function javascript_validation() {
      return false;
    }

    //If this particular payment method has an alternate checkout method or if needs some initialisation 
    //(such as getting security token from the payment processor) it should be defined here
    //This option will be presented in the checkout screen
    //Called by shopping_cart.php
    //This function is listed here only for completeness 
//    function checkout_initialization_method() {
//      return false;
//    }
    
    //Returns the elements required for the shopping cart to construct and display the payment form to the card holder
    //This form will be displayed below the payment option name
    //For HPP this function can simply return the module id and the title
    //This function is called by the shopping cart while listing the available payment options
    function selection() {
      return array(
        'id' => $this->code,
        'module' => $this->public_title
      );
    }

    //Function to validate the credit card data before moving on to the confirmation page
    //For HPP this function will return false as there is nothing to validate
    //This function is called before checkout confirmation page is displayed
    function pre_confirmation_check() {
      if ($errors = $this->config_errors()) {
        //$errors is an array. 
        //Pre-confirmation check is called by checkout so just display the first error. No need to dispaly all the errors 
        tep_redirect(tep_href_link(FILENAME_CHECKOUT_PAYMENT, 'payment_error=' . $this->code . '&error=' . $errors[0], 'NONSSL'));
      }
    }

    //Display the payment information (abbreviated CC number, Expiry, etc) in the
    //order confirmation page
    //In HPP this will just be the psyment option name and probably some other instructions
    //This function is called by the order confirmation page 
    function confirmation() {
      $confirmation = array('title' => $this->public_title,
                            'fields' => array(
                                              array('title' => '<b>'.MODULE_PAYMENT_NAB_TRANSACT_HPP_TEXT_PAYMENT_SECURITY_HEADING.'</b>',
                                                    'field' => MODULE_PAYMENT_NAB_TRANSACT_HPP_TEXT_PAYMENT_SECURITY_INSTRUCTIONS),
                                             )
                           );
      return $confirmation;
    }

    //Function to construct the payment button
    //This function will be used to construct the hidden fields (such as account id, order amount, etc)
    //specific to the gateway
    function process_button() {
      global $order, $customer_id, $nt_hpp_payment_reference, $customer_ip, $order_total_modules;

      //Record customer's ip in to the session
      $customer_ip = trim($_SERVER['REMOTE_ADDR']);
      tep_session_register('customer_ip');

      if (!tep_session_is_registered('nt_hpp_payment_reference')) {
        $nt_hpp_payment_reference = $customer_id . '-' . date("YmdGis");
        tep_session_register('nt_hpp_payment_reference');
      }
      
      $query_string = tep_session_name().'='.tep_session_id()
                    . '&' . MODULE_PAYMENT_NAB_TRANSACT_QS_PARAM_NAME_BANK_REF . '='
                    . '&' . MODULE_PAYMENT_NAB_TRANSACT_QS_PARAM_NAME_CARD_TYPE .'='
                    . '&' . MODULE_PAYMENT_NAB_TRANSACT_QS_PARAM_NAME_AMNT_PAID .'='
                    . '&' . MODULE_PAYMENT_NAB_TRANSACT_QS_PARAM_NAME_REF_NUM .'='
                    . '&' . MODULE_PAYMENT_NAB_TRANSACT_QS_PARAM_NAME_CUST_IP .'='
                    ;
      
      $gateway_vars = Array();    
      $gateway_vars['vendor_name'] = MODULE_PAYMENT_NAB_TRANSACT_HPP_CLIENT_ID;
      $gateway_vars['print_zero_qty'] = 'false';
      $gateway_vars['receipt_address'] = $order->customer['email_address'];
      $gateway_vars['payment_reference'] = $nt_hpp_payment_reference;
      $gateway_vars['return_link_url'] = tep_href_link(FILENAME_CHECKOUT_SUCCESS, tep_session_name().'='.tep_session_id());
      $gateway_vars['reply_link_url'] = tep_href_link(FILENAME_CHECKOUT_PROCESS, $query_string); 
      unset($query_string);
      
      if (DISPLAY_PRICE_WITH_TAX == 'true') {
        $gateway_vars['gst_added'] = 'true' ;
      }
      
      //If the merchant has set the payment_alert email via the module pass that email to NAB Transact
      if(strlen(trim(MODULE_PAYMENT_NAB_TRANSACT_HPP_PAYMENT_ALERT_ADDRESS)) != 0){                          
        $gateway_vars['payment_alert'] = MODULE_PAYMENT_NAB_TRANSACT_HPP_PAYMENT_ALERT_ADDRESS;
      }
      //If the Reply Link Text has been configured via the Admin pass the value
      if(strlen(trim(MODULE_PAYMENT_NAB_TRANSACT_HPP_RETURN_LINK_TEXT)) != 0){
        $gateway_vars['return_link_text'] = MODULE_PAYMENT_NAB_TRANSACT_HPP_RETURN_LINK_TEXT;
      }
      
      //Build products fields    
      foreach($order->products as $key => $value) {
        $product_name = $value['name'];
        if (strlen(trim($value['model'])) > 0) {
          $product_name .= '-'.MODULE_PAYMENT_NAB_TRANSACT_HPP_PROD_DESC_MODEL.':'.$value['model'];;
        }
        if (array_key_exists('attributes', $value)) {
          $product_name .= ' (';
          foreach($value['attributes'] as $attrib_key => $attrib_value) {
            $product_name .= $attrib_value['option'] . ':' . $attrib_value['value'] . ', ';
          }
          //trim the trailing ', '
          $product_name = substr($product_name, 0, -2);
          $product_name .= ')';
        }
        //Remove $ char if the product name has it. For some strange reason HPP displays blank page if there is $
        $product_name = str_replace('$', '', $product_name);
        $product_qty = $value['qty'];
        //Make sure the unit price has 2 decimals and no thousands seperator
        $product_price = number_format((float)tep_add_tax($value['final_price'], $value['tax']), 2, '.', '');
        $gateway_vars[$product_name] = $product_qty.','.$product_price;
      }
      
      //Shipping Cost
      //Shipping cost may be set to $0.00 if free shipping is enabled
      $shipping_desc = $order->info['shipping_method'];
      //If shipping desc is not set (will be the case for virtual products) skip this
      if (strlen($shipping_desc) > 0) {
        if (strpos($shipping_desc, '<') !== false) {
          $shipping_desc = '';
        }
        $gateway_vars[MODULE_PAYMENT_NAB_TRANSACT_HPP_PROD_DESC_SHIPPING.': '.$shipping_desc] = '1,'.number_format((float)$order->info['shipping_cost'], 2, '.', '');
      }
      unset($shipping_desc);
      
      //Taxes
      foreach($order->info['tax_groups'] as $name => $value) {
        if ((float)$value > 0){
          if (DISPLAY_PRICE_WITH_TAX == 'true'){
            //If the prices are displayed with tax add the description of the tax as a product with $0 value
            //This will make sure the tax details are displayed in the payment page
            //$0 value will make sure the tax is not added AGAIN to the order total
            $gateway_vars[number_format((float)$value, 2, '.', '').' '.$name.' '.MODULE_PAYMENT_NAB_TRANSACT_HPP_PROD_DESC_TAX_INCLUDED] = '1,0';
          } else {
            $gateway_vars[MODULE_PAYMENT_NAB_TRANSACT_HPP_PROD_DESC_TAX.': '.$name] = '1,'.number_format((float)$value, 2, '.', '');
          }
        }
      }
      //Go through the order totals
      if (is_array($order_total_modules->modules)) {
        reset($order_total_modules->modules);
        while (list(, $value) = each($order_total_modules->modules)) {
          $class = substr($value, 0, strrpos($value, '.'));
          $ot_object = $GLOBALS[$class];
          //if ($ot_object->enabled) {
          //For some reason ot_discount_coupon never returns true to this
            for ($i=0, $size = sizeof($ot_object->output); $i<$size; $i++) {
              $ot_code = $ot_object->code;
              $ot_object_output = $ot_object->output[$i];
              $ot_title = substr($ot_object_output['title'], 0, -1);
              $ot_value = (float)$ot_object_output['value'];
              //If it's a discount a deduction module make the value -ve
              if (($ot_object->deduction > 0) or (strcasecmp($ot_code, 'ot_discount_coupon') == 0) or $ot_object->credit_class) {
                $ot_value = (float)('-'.$ot_value);
              }
              switch (strtoupper($ot_code)) {
                //Ignore the Tax and Shipping as we have already added them
                //Why we are not adding them here?
                //If the ot_tax and ot_shipping modules are removed from Modules->Order Totals in Admin 
                //the these costs will not show up in the order totals (hence will not be 
                //passed to the gateway). It's better to get the Tax and Shipping from $order->info[]
                case strtoupper('ot_tax'):
                  break;
                case strtoupper('ot_shipping'):
                  break;
                //Exclude Sub Total and Total modules from the calculations
                case strtoupper('ot_subtotal'):
                  break;
                case strtoupper('ot_total'):
                  break;
                //Add any other totals to the gateway variables           
                default:
                  $gateway_vars[$ot_title] = '1,'.number_format($ot_value, 2, '.', '');
              }
            }
          //}
        }
      }
      
      //Build the information fields
      $information_fields = array();
      if (MODULE_PAYMENT_NAB_TRANSACT_HPP_DISPLAY_CUSTOMER_DETAILS == MODULE_PAYMENT_NAB_TRANSACT_CONST_YES) {
        $information_fields['Customer_Name'] = $order->customer['firstname'].' '.$order->customer['lastname'];
        $information_fields['Customer_Company_Name'] = $order->customer['company'];
        $information_fields['Customer_Street'] = $order->customer['street_address'];
        $information_fields['Customer_Suburb'] = $order->customer['suburb'];
        $information_fields['Customer_City'] = $order->customer['city'];
        $information_fields['Customer_State'] = $order->customer['state'];
        $information_fields['Customer_Post_Code'] = $order->customer['postcode'];
        $information_fields['Customer_Country'] = $order->customer['country']['title'];
        $information_fields['Customer_Email'] = $order->customer['email_address'];
        $information_fields['Customer_Phone'] = $order->customer['telephone'];
      }
      
      //Check if the Shiiping details should be displayed and the products are not virtual 
      if ((MODULE_PAYMENT_NAB_TRANSACT_HPP_DISPLAY_SHIPPING_DETAILS == MODULE_PAYMENT_NAB_TRANSACT_CONST_YES) && ($order->content_type != 'virtual')) {
        $information_fields['Shipping_Name'] = $order->delivery['firstname'].' '.$order->delivery['lastname'];
        $information_fields['Shipping_Company_Name'] = $order->delivery['company'];
        $information_fields['Shipping_Street'] = $order->delivery['street_address'];
        $information_fields['Shipping_Suburb'] = $order->delivery['suburb'];
        $information_fields['Shipping_City'] = $order->delivery['city'];
        $information_fields['Shipping_State'] = $order->delivery['state'];
        $information_fields['Shipping_Post_Code'] = $order->delivery['postcode'];
        $information_fields['Shipping_Country'] = $order->delivery['country']['title'];
      }
      //Check if the Billing details should be displayed
      if (MODULE_PAYMENT_NAB_TRANSACT_HPP_DISPLAY_BILLING_DETAILS == MODULE_PAYMENT_NAB_TRANSACT_CONST_YES) {
        $information_fields['Billing_Name'] = $order->billing['firstname'].' '.$order->billing['lastname'];
        $information_fields['Billing_Company_Name'] = $order->billing['company'];
        $information_fields['Billing_Street'] = $order->billing['street_address'];
        $information_fields['Billing_Suburb'] = $order->billing['suburb'];
        $information_fields['Billing_City'] = $order->billing['city'];
        $information_fields['Billing_State'] = $order->billing['state'];
        $information_fields['Billing_Post_Code'] = $order->billing['postcode'];
        $information_fields['Billing_Country'] = $order->billing['country']['title'];
      }
      $comment = $order->info['comments'];
      //Check if the buyer's comment should be displayed and not empty
      if ((MODULE_PAYMENT_NAB_TRANSACT_HPP_DISPLAY_BUYER_COMMENT == MODULE_PAYMENT_NAB_TRANSACT_CONST_YES) && (strlen(trim($comment)) != 0)) {
        $information_fields['Comments'] = $comment;
      }
      unset($comment);
      
      if (count($information_fields) > 0) {
        $gateway_vars = array_merge($gateway_vars, $information_fields);
        $gateway_vars['information_fields'] = implode(',', array_keys($information_fields));
      }
      unset($information_fields);
      
      //The following will remove the co-ordinates of the submit button being picked up as products
      $gateway_vars['information_fields'] .= ',x,y';
      $gateway_vars['hidden_fields'] = 'x,y';
      //Go through the gateway variables and construct the hidden fields
      $process_button_string = '';
      foreach($gateway_vars as $name => $value) {
        //Make sure the there are no $ char is in the field name. $ char makes NT brings a blank page
        $process_button_string .= chr(13) . tep_draw_hidden_field(str_replace("$", "", $name), $value);
      }
      $process_button_string .= chr(13);
      unset($gateway_vars);
      
      return $process_button_string;
    }

    //This function is called by checkout_process.php before recording the order (processing the order)
    //This function will/should only be called after the payment has been successfully processed
    //This function should be triggered by reply_link_ur being called by NAB Transact server 
    function before_process() {
      global $order, $customer_ip, $amount_paid;

      $amount_paid = $_GET[MODULE_PAYMENT_NAB_TRANSACT_QS_PARAM_NAME_AMNT_PAID];
      //Check if the ip address calling this process is same as the previously (in process_button) recorded
      //customer's IP.
      //This check will ensure the customer is not just pasting the reply_link_url in the browser
      //and spoofing a success result
      //Ideally it's best to check the the remote ip is NT server's ip and remote host is the host name of NT server
      //This is not possible as NT server does not return the host name and returns multiple ip addresses
      if ($_SERVER['REMOTE_ADDR'] != $customer_ip || empty($customer_ip)){        
        //Just update the order status
        //Can't write the payment details to the order history yet as the order hasn't been added to the db yet. hence order id doesn't exist        
        if(MODULE_PAYMENT_NAB_TRANSACT_HPP_ORDER_STATUS_ID > 0 && round((float)$order->info['total'],2) == round((float)$amount_paid, 2)) {
          $order->info['order_status'] = MODULE_PAYMENT_NAB_TRANSACT_HPP_ORDER_STATUS_ID;
        }
      //if the customer has copied & pasted the URL redirect the payment page with an error message
      } else {
        tep_redirect(tep_href_link(FILENAME_CHECKOUT_PAYMENT, 'payment_error=' . $this->code . '&error=' . MODULE_PAYMENT_NAB_TRANSACT_ERR_UNAUTH_CHECKOUT_CALL, 'SSL'));
      }
    }

    //This function is called by checkout_process.php after the checkout process is complete and 
    //before the redirecting to checkout_success
    function after_process() {
      global $order, $insert_id, $nt_hpp_payment_reference, $customer_ip, $amount_paid;
      
      //Write the payment specific values to the order history
      if ($_SERVER['REMOTE_ADDR'] != $customer_ip || empty($customer_ip)){
          $order_status = $order->info['order_status'];
          $payment_status = 'NOT PAID/PARTIALLY PAID';
          //If the full amount has been paid update the order status
          //otherwise leave the status as it is
          if (round((float)$order->info['total'],2) == round((float)$amount_paid, 2)) {
            if(MODULE_PAYMENT_NAB_TRANSACT_HPP_ORDER_STATUS_ID > 0) {
              $order_status = MODULE_PAYMENT_NAB_TRANSACT_HPP_ORDER_STATUS_ID;
            }
            $payment_status = 'PAID IN FULL';
          }
          $sql_data_array = array(
            'orders_id' => $insert_id, 
            'orders_status_id' => $order_status, 
            'date_added' => 'now()', 
            'customer_notified' => 0, 
            'comments' => 
              '***'.MODULE_PAYMENT_NAB_TRANSACT_HPP_PAY_INFO_HEADING.'***'.chr(10).
              MODULE_PAYMENT_NAB_TRANSACT_HPP_PAY_INFO_STATUS.': '.$payment_status.' ('.strtoupper($this->in_test_mode() ? MODULE_PAYMENT_NAB_TRANSACT_CONST_TEST : MODULE_PAYMENT_NAB_TRANSACT_CONST_LIVE).' PAYMENT)'.chr(10).
              MODULE_PAYMENT_NAB_TRANSACT_HPP_PAY_INFO_AMOUNT.': '.$amount_paid .
              MODULE_PAYMENT_NAB_TRANSACT_HPP_PAY_INFO_CARD.': '.urldecode($_GET[MODULE_PAYMENT_NAB_TRANSACT_QS_PARAM_NAME_CARD_TYPE]).chr(10).
              MODULE_PAYMENT_NAB_TRANSACT_HPP_PAY_INFO_TXN_REF.'#: '.$_SESSION['nt_hpp_payment_reference'].':'.urldecode($_GET[MODULE_PAYMENT_NAB_TRANSACT_QS_PARAM_NAME_REF_NUM]).chr(10).
              MODULE_PAYMENT_NAB_TRANSACT_HPP_PAY_INFO_BANK_REF.'#: '.urldecode($_GET[MODULE_PAYMENT_NAB_TRANSACT_QS_PARAM_NAME_BANK_REF]).chr(10)
            );
          tep_db_perform(TABLE_ORDERS_STATUS_HISTORY, $sql_data_array);
          //unregister the session variables
          tep_session_unregister('nt_hpp_payment_reference');
          tep_session_unregister('customer_ip');
      } else {
        tep_redirect(tep_href_link(FILENAME_CHECKOUT_PAYMENT, 'payment_error=' . $this->code . '&error=' . MODULE_PAYMENT_NAB_TRANSACT_ERR_UNAUTH_CHECKOUT_CALL, 'SSL'));
      }
    }
    
    //Handle the payment gateway errors
    //Called by by checkout_payment after after_process() function call
    //The paramaters to the function call are added to enable this function to be used by the Admin Area
    //to handle the configuration errors
    function get_error($err = '', $msg_type = 'long') {
      global $HTTP_GET_VARS, $nt_hpp_payment_reference;
      if (strlen($err) > 0) {
        $err_code = $err;
      } else {
        $err_code = trim($HTTP_GET_VARS['error']);
      }
      if (strlen($err_code) > 0) {
        switch ($err_code) {
          case MODULE_PAYMENT_NAB_TRANSACT_ERR_UNAUTH_CHECKOUT_CALL:
            $error = MODULE_PAYMENT_NAB_TRANSACT_HPP_ERROR_UNAUTH_CHECKOUT_CALL.' '.$nt_hpp_payment_reference;
            break;
          case MODULE_PAYMENT_NAB_TRANSACT_ERR_INVALID_MERCH_ID:
            $error = ($msg_type != 'long' ? MODULE_PAYMENT_NAB_TRANSACT_HPP_ERROR_INVALID_MERCH_ID_SHORT : MODULE_PAYMENT_NAB_TRANSACT_HPP_ERROR_INVALID_MERCH_ID);
            break;
          default:
            $error = MODULE_PAYMENT_NAB_TRANSACT_HPP_ERROR_UNKNOWN_ERROR.' '.$nt_hpp_payment_reference;
        }
      } else {
        $error = MODULE_PAYMENT_NAB_TRANSACT_HPP_ERROR_UNKNOWN_ERROR.' '.$nt_hpp_payment_reference;
      }
      return array('title' => MODULE_PAYMENT_NAB_TRANSACT_HPP_TEXT_ERROR_TITLE,
                   'error' => $error);
    }
    
    //Check the status of this module
    //This function called by the cart admin to check the installation status of this module 
    function check() {
      if (!isset($this->_check)) {
      	//Check if there any configuration entries in db for this module
        $this->_check = tep_db_num_rows(tep_db_query("select configuration_value from " . TABLE_CONFIGURATION . " where configuration_key = 'MODULE_PAYMENT_NAB_TRANSACT_HPP_ENABLED'"));
      }
      //if there are any entries in the db the module is installed (return true)
      return ($this->_check > 0);
    }

    //Install the modules
    function install() {
      define ('CONFIG_GROUP_ID', '6');
     	//An auto incrementing variable to be used for Configuration fields sort_order  
    	$index = 0;
      tep_db_query("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, set_function, date_added) values ('Enable NAB Transact module', 'MODULE_PAYMENT_NAB_TRANSACT_HPP_ENABLED', '" . MODULE_PAYMENT_NAB_TRANSACT_CONST_YES . "', 'Do you want to use NAB Transact to process credit card payments?', '".CONFIG_GROUP_ID."', '" . $index++ . "', 'tep_cfg_select_option(array(\'" . MODULE_PAYMENT_NAB_TRANSACT_CONST_YES . "\', \'" . MODULE_PAYMENT_NAB_TRANSACT_CONST_NO . "\'), ', now())");
      tep_db_query("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, date_added) values ('Display order', 'MODULE_PAYMENT_NAB_TRANSACT_HPP_SORT_ORDER', '0', 'The available payment options will be listed to your customer during the checkout process. This value determines listing order.<br>Where do you want this payment option to appear (Lowest is displayed first)?', '".CONFIG_GROUP_ID."', '" . $index++ . "', now())");
      tep_db_query("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, set_function, date_added) values ('NAB Transact mode', 'MODULE_PAYMENT_NAB_TRANSACT_HPP_GATEWAY_MODE', '" . MODULE_PAYMENT_NAB_TRANSACT_CONST_TEST . "', 'Do you want to connect to test or live NAB Transact system?<br><span class=\"messageStackWarning\">Please make sure this is set to \'" . MODULE_PAYMENT_NAB_TRANSACT_CONST_LIVE . "\' when you want to start accepting real payments.</span>', '".CONFIG_GROUP_ID."', '" . $index++ . "', 'tep_cfg_select_option(array(\'" . MODULE_PAYMENT_NAB_TRANSACT_CONST_TEST . "\', \'" . MODULE_PAYMENT_NAB_TRANSACT_CONST_LIVE . "\'), ', now())");
      tep_db_query("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, date_added) values ('NAB Transact Client Id', 'MODULE_PAYMENT_NAB_TRANSACT_HPP_CLIENT_ID', '', 'Enter your <u>7 character</u> NAB Client Id.</br>You will find this id in the Account Activation Email.</br>If you are still waiting for the Account Activation Email or evaluating NAB Transact gateway before signing up, contact NAB Transact Service Centre.', '".CONFIG_GROUP_ID."', '" . $index++ . "', now())");
      tep_db_query("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, set_function, use_function, date_added) values ('Order Status', 'MODULE_PAYMENT_NAB_TRANSACT_HPP_ORDER_STATUS_ID', '0', 'Once the customer has made a successful payment what do you want to be the status of the order?', '".CONFIG_GROUP_ID."', '" . $index++ . "', 'tep_cfg_pull_down_order_statuses(', 'tep_get_order_status_name', now())");
      tep_db_query("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, date_added) values ('Payment Alert Address', 'MODULE_PAYMENT_NAB_TRANSACT_HPP_PAYMENT_ALERT_ADDRESS', '".STORE_OWNER_EMAIL_ADDRESS."', 'When a payment is successfully processed NAB Transact will send an email to the merchant. Which email address you want these emails sent to?<br>If you leave blank, the address specified in your NAB Transact account will be used', '".CONFIG_GROUP_ID."', '" . $index++ . "', now())");
      tep_db_query("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, date_added) values ('Return Link Text', 'MODULE_PAYMENT_NAB_TRANSACT_HPP_RETURN_LINK_TEXT', 'Return to ".STORE_NAME."', 'A link will be displayed in the NAB Transact Receipt page, which will allow the customer to click and return back to your website.<br>This text will be displayed along with the link.', '".CONFIG_GROUP_ID."', '" . $index++ . "', now())");
      tep_db_query("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, set_function, date_added) values ('Display Customer Details', 'MODULE_PAYMENT_NAB_TRANSACT_HPP_DISPLAY_CUSTOMER_DETAILS', '" . MODULE_PAYMENT_NAB_TRANSACT_CONST_YES . "', 'Do you want to display the customer details in the NAB Transact payment page?', '".CONFIG_GROUP_ID."', '" . $index++ . "', 'tep_cfg_select_option(array(\'" . MODULE_PAYMENT_NAB_TRANSACT_CONST_YES . "\', \'" . MODULE_PAYMENT_NAB_TRANSACT_CONST_NO . "\'), ', now())");
      tep_db_query("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, set_function, date_added) values ('Display Shipping Details', 'MODULE_PAYMENT_NAB_TRANSACT_HPP_DISPLAY_SHIPPING_DETAILS', '" . MODULE_PAYMENT_NAB_TRANSACT_CONST_YES . "', 'Do you want to display the shipping details in the NAB Transact payment page?</br>If the cart contains only virtual products, the shipping address will not be displayed even if this is set to yes', '".CONFIG_GROUP_ID."', '" . $index++ . "', 'tep_cfg_select_option(array(\'" . MODULE_PAYMENT_NAB_TRANSACT_CONST_YES . "\', \'" . MODULE_PAYMENT_NAB_TRANSACT_CONST_NO . "\'), ', now())");
      tep_db_query("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, set_function, date_added) values ('Display Billing Details', 'MODULE_PAYMENT_NAB_TRANSACT_HPP_DISPLAY_BILLING_DETAILS', '" . MODULE_PAYMENT_NAB_TRANSACT_CONST_YES . "', 'Do you want to display the billing details in the NAB Transact payment page?', '".CONFIG_GROUP_ID."', '" . $index++ . "', 'tep_cfg_select_option(array(\'" . MODULE_PAYMENT_NAB_TRANSACT_CONST_YES . "\', \'" . MODULE_PAYMENT_NAB_TRANSACT_CONST_NO . "\'), ', now())");
      tep_db_query("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, set_function, date_added) values ('Display Buyer\'s Comments', 'MODULE_PAYMENT_NAB_TRANSACT_HPP_DISPLAY_BUYER_COMMENT', '" . MODULE_PAYMENT_NAB_TRANSACT_CONST_YES . "', 'Do you want to display the comments entered by the buyer in the NAB Transact payment page?', '".CONFIG_GROUP_ID."', '" . $index++ . "', 'tep_cfg_select_option(array(\'" . MODULE_PAYMENT_NAB_TRANSACT_CONST_YES . "\', \'" . MODULE_PAYMENT_NAB_TRANSACT_CONST_NO . "\'), ', now())");
      tep_db_query("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, date_added) values ('NAB Transact HPP Test URL', 'MODULE_PAYMENT_NAB_TRANSACT_HPP_TEST_URL', '" . MODULE_PAYMENT_NAB_TRANSACT_HPP_CONST_TEST_URL . "', 'The URL of the NAB Transact TEST Hosted Payment Page', '".CONFIG_GROUP_ID."', '" . $index++ . "', now())");
      tep_db_query("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, date_added) values ('NAB Transact HPP Live URL', 'MODULE_PAYMENT_NAB_TRANSACT_HPP_LIVE_URL', '" . MODULE_PAYMENT_NAB_TRANSACT_HPP_CONST_LIVE_URL . "', 'The URL of the NAB Transact LIVE Hosted Payment Page', '".CONFIG_GROUP_ID."', '" . $index++ . "', now())");
      tep_db_query("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, set_function, date_added) values ('Advanced Configuration".tep_black_line()."', 'MODULE_PAYMENT_NAB_TRANSACT_HPP_ADVANCED_CONFIG', '', 'Do not modify the fields below unless you have specifically been Advised by NAB Transact.<br/><span class=\"messageStackWarning\">Modifying these values may prevent this module from working.</span>', '".CONFIG_GROUP_ID."', '" . $index++ . "', 'tep_draw_separator(', now())");
      tep_db_query("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, use_function, set_function, date_added) values ('Limit to Zone', 'MODULE_PAYMENT_NAB_TRANSACT_HPP_ZONE', '', 'Do you want to make this payment option available only to a selected zone?<br/>To make this payment option available to all customers select ".TEXT_NONE.". Otherwise, this payment option will be made available only to the customer from the selected zone','".CONFIG_GROUP_ID."', '" . $index++ . "', 'tep_get_zone_class_title', 'tep_cfg_pull_down_zone_classes(', now())");
    }

    //Uninstall the module
    function remove() {
      tep_db_query("delete from " . TABLE_CONFIGURATION . " where configuration_key in ('" . implode("', '", $this->keys()) . "')");
    }

    function keys() {
      return array(
        'MODULE_PAYMENT_NAB_TRANSACT_HPP_ENABLED',
        'MODULE_PAYMENT_NAB_TRANSACT_HPP_SORT_ORDER',
        'MODULE_PAYMENT_NAB_TRANSACT_HPP_ZONE',
        'MODULE_PAYMENT_NAB_TRANSACT_HPP_GATEWAY_MODE',
        'MODULE_PAYMENT_NAB_TRANSACT_HPP_CLIENT_ID',
        'MODULE_PAYMENT_NAB_TRANSACT_HPP_ORDER_STATUS_ID',
        'MODULE_PAYMENT_NAB_TRANSACT_HPP_PAYMENT_ALERT_ADDRESS',
        'MODULE_PAYMENT_NAB_TRANSACT_HPP_RETURN_LINK_TEXT',
        'MODULE_PAYMENT_NAB_TRANSACT_HPP_DISPLAY_CUSTOMER_DETAILS',
        'MODULE_PAYMENT_NAB_TRANSACT_HPP_DISPLAY_SHIPPING_DETAILS',
        'MODULE_PAYMENT_NAB_TRANSACT_HPP_DISPLAY_BILLING_DETAILS',
        'MODULE_PAYMENT_NAB_TRANSACT_HPP_DISPLAY_BUYER_COMMENT',
        'MODULE_PAYMENT_NAB_TRANSACT_HPP_ADVANCED_CONFIG',
        'MODULE_PAYMENT_NAB_TRANSACT_HPP_TEST_URL',
        'MODULE_PAYMENT_NAB_TRANSACT_HPP_LIVE_URL',
      );
    }
    
    //Internal function to test if the module is in test mode
    function in_test_mode() {
      return (MODULE_PAYMENT_NAB_TRANSACT_HPP_GATEWAY_MODE == MODULE_PAYMENT_NAB_TRANSACT_CONST_TEST);
    }
    
    //Internal function to check if there are any errors with the module configuration
    function config_errors() {
      $errors = array();
      if (strlen(MODULE_PAYMENT_NAB_TRANSACT_HPP_CLIENT_ID) < MODULE_PAYMENT_NAB_TRANSACT_CLIENT_ID_LENGTH) {
        $errors[] = MODULE_PAYMENT_NAB_TRANSACT_ERR_INVALID_MERCH_ID;
      }
      return (count($errors) > 0 ? $errors : false);
    }
    
    //Internal function to check if there are any warning for this module
    function warnings() {
      $warnings = array();
      //Define the possible local IPs. These will be know private IPs
      //Wild cards (*,?) can be used 
      $localhosts = array(
        '127.0.0.1',
        '192.168.*.*',
        '10.*.*.*',
        '172.16.*.*',
        '172.17.*.*',
        '172.18.*.*',
        '172.19.*.*',
        '172.20.*.*',
        '172.21.*.*',
        '172.22.*.*',
        '172.23.*.*',
        '172.24.*.*',
        '172.25.*.*',
        '172.26.*.*',
        '172.27.*.*',
        '172.28.*.*',
        '172.29.*.*',
        '172.30.*.*',
        '172.31.*.*'
      );
      if ($this->in_test_mode()) {
        $warnings[] = MODULE_PAYMENT_NAB_TRANSACT_HPP_WARN_TEST_MODE;
      }
      $server_ip = $_SERVER['SERVER_ADDR'];
      $match = false;
      foreach ($localhosts as $local_ip) {
        $match = preg_match('/^'.str_replace(array('\*','\?'),array('(.*?)','[0-9]'), preg_quote($local_ip)).'$/', $server_ip);
        if ($match) {
          $warnings[] = MODULE_PAYMENT_NAB_TRANSACT_HPP_WARN_INVALID_HOST;   
          break;  
        }
      }
      return (count($warnings) > 0 ? $warnings : false);
    }      
  }
?>